/**********
 * Copyright (c) 2003-2005 Greg Parker.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY GREG PARKER ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **********/

#include "includes.h"
#include "formutils.h"
#include "data/prefs.h"
#include "data/connectionlist.h"
#include "rsrc/rsrc.h"
#include "forms/resize.h"

#include "loginform.h"


static Boolean ValidateLoginForm(void) SSH_SEGMENT;
static Boolean SaveLoginForm(void) SSH_SEGMENT;
static void LoadEmptyLoginForm(void) SSH_SEGMENT;


static Boolean ValidateLoginForm(void)
{
    return true; // fixme
}


// returns FALSE if save failed for some reason
// if save fails, caller shouldn't connect
static Boolean SaveLoginForm(void)
{
    FieldPtr protocolFld = PrvGetObjectByID(LoginFormProtocolFieldID);
    FieldPtr hostFld = PrvGetObjectByID(LoginFormHostFieldID);
    FieldPtr portFld = PrvGetObjectByID(LoginFormPortFieldID);
    FieldPtr userFld = PrvGetObjectByID(LoginFormUsernameFieldID);

   // protocol
    

    if (FldDirty(protocolFld) || FldDirty(hostFld)  ||  FldDirty(portFld)  ||  FldDirty(userFld)) {
		return ConnectionListWriteSelectedRecord(protocolFld,hostFld, portFld, userFld);
    } else {
        return true;
    }
}


static void LoadEmptyLoginForm(void)
{
    // set port field to default value; leave others empty
    char *str;

    // terminal type
    FieldPtr protocolFld = PrvGetObjectByID(LoginFormProtocolFieldID);

    PrvSetFieldToValueByID(LoginFormProtocolFieldID, "telnet");
	FldReleaseFocus(protocolFld);
    FldCompactText(protocolFld);
    str = FldGetTextPtr(protocolFld);

	if(0 == strncmp(str, "telnet", strlen("telnet")) )
	{
	    PrvSetFieldToValueByID(LoginFormPortFieldID, "23");
		PrvHideObjectByID(LoginFormUsernameFieldID);
	}
	else
	{
	    PrvSetFieldToValueByID(LoginFormPortFieldID, "22");
		PrvShowObjectByID(LoginFormUsernameFieldID);
	}
    PrvSetFocusByID(LoginFormHostFieldID);
}


Boolean LoginFormHandleEvent(EventPtr e)
{
    FormPtr frmP = FrmGetActiveForm();

    if (ResizeHandleEvent(e)) return true;

    switch (e->eType) {
    case frmOpenEvent: {
        // "Connect" with nothing selected, or "New..."
        LoadEmptyLoginForm();
        FrmDrawForm(frmP);
        return true;
    }
    case popSelectEvent: {
        if (e->data.popSelect.listID == LoginFormProtocolListID) {
            FieldPtr fld;
            char *str = LstGetSelectionText(e->data.popSelect.listP,
                                            e->data.popSelect.selection);
            PrvSetFieldToValueByID(LoginFormProtocolFieldID, str);
            fld = PrvGetObjectByID(LoginFormProtocolFieldID);
            FldEraseField(fld);
            FldDrawField(fld);

			if(0 == strncmp(str, "telnet", strlen("telnet")) )
			{
				PrvSetFieldToValueByID(LoginFormPortFieldID, "23");
				PrvShowObjectByID(LoginFormPortFieldID);
				PrvHideObjectByID(LoginFormUsernameFieldID);
			}
			else
			{
				PrvSetFieldToValueByID(LoginFormPortFieldID, "22");
				PrvShowObjectByID(LoginFormPortFieldID);
				PrvShowObjectByID(LoginFormUsernameFieldID);
			}
			PrvSetFocusByID(LoginFormHostFieldID);
        }

        // DON'T let popup trigger see this event and change its name
        return true; 
    }
    case ctlSelectEvent:
        switch (e->data.ctlSelect.controlID) {
        case LoginFormConnectButtonID:
            // make sure form data is good enough to use for a connection
            if (!ValidateLoginForm()) break;
            // Clear selection as late as possible
            ConnectionListClearSelection();
            if (SaveLoginForm()) {
                // login info saved to ConnectionList selected record
                // go to TerminalForm which will initiate connection
                FrmReturnToForm(0);
                FrmGotoForm(TerminalFormID);
            } else {
                // failed to save login info
                // fixme
                FrmReturnToForm(0);
            }
            break;

        case LoginFormCancelButtonID:
            // return to main form
            FrmReturnToForm(0);
            break;

        case LoginFormSaveButtonID:
            // save login info, return to main form
            // Don't validate - let the user enter anything
            ConnectionListClearSelection();
            if (!SaveLoginForm()) {
                // failed to save login info
                // fixme
            }
            FrmReturnToForm(0);
            break;
        case LoginFormProtocolTriggerID: {
            FieldPtr fld;
            ListPtr lst;
            char *fldStr;
            int i, count;

            fld = PrvGetObjectByID(LoginFormProtocolFieldID);
            fldStr = FldGetTextPtr(fld);
            lst = PrvGetObjectByID(LoginFormProtocolListID);
            count = LstGetNumberOfItems(lst);
            for (i = 0; i < count; i++) {
                char *lstStr = LstGetSelectionText(lst, i);
                if (0 == strcmp(lstStr, fldStr)) {
                    LstSetSelection(lst, i);
                    break;
                }
            }
            if (i == count) {
                LstSetSelection(lst, noListSelection);
            }
            
            return false; // continue with ordinary processing
        }
/*			
        case LoginFormTelnetCheckboxID:
			{
	        PrefsPutInt(prefProtocol, protocolTelnet);
			PrvSetFieldToValueByID(LoginFormPortFieldID, "23");
			PrvShowObjectByID(LoginFormPortFieldID);
			PrvHideObjectByID(LoginFormUsernameFieldID);
            break;
        	}
        case LoginFormSsh2CheckboxID:
			{
	        PrefsPutInt(prefProtocol, protocolSsh2);
			PrvSetFieldToValueByID(LoginFormPortFieldID, "22");
			PrvShowObjectByID(LoginFormPortFieldID);
			PrvShowObjectByID(LoginFormUsernameFieldID);
            break;
        	}*/
        default:
            break;
        }

        return true;

    case frmCloseEvent:
        return false;

    case usrSetFocusEvent:
        PrvReallySetFocus(frmP, e);
        return true;

    default: 
        return false;
    }
}
